@@ -98,6 +98,9 @@
 #ifdef LIBXML_C14N_ENABLED
 #include <libxml/c14n.h>
 #endif
+#ifdef LIBXML_OUTPUT_ENABLED
+#include <libxml/xmlsave.h>
+#endif
 
 #ifndef XML_XML_DEFAULT_CATALOG
 #define XML_XML_DEFAULT_CATALOG "file:///var/db/xmlcatalog"
@@ -133,6 +136,7 @@ static int nowrap = 0;
 static int format = 0;
 static const char *output = NULL;
 static int compress = 0;
+static int oldout = 0;
 #endif /* LIBXML_OUTPUT_ENABLED */
 #ifdef LIBXML_VALID_ENABLED
 static int valid = 0;
@@ -205,6 +209,11 @@ static int sax = 0;
  *									*
  ************************************************************************/
 #define MAX_PATHS 64
+#ifdef _WIN32
+# define PATH_SEPARATOR ';'
+#else
+# define PATH_SEPARATOR ':'
+#endif
 static xmlChar *paths[MAX_PATHS + 1];
 static int nbpaths = 0;
 static int load_trace = 0;
@@ -221,10 +230,10 @@ void parsePath(const xmlChar *path) {
 	    return;
 	}
 	cur = path;
-	while ((*cur == ' ') || (*cur == ':'))
+	while ((*cur == ' ') || (*cur == PATH_SEPARATOR))
 	    cur++;
 	path = cur;
-	while ((*cur != 0) && (*cur != ' ') && (*cur != ':'))
+	while ((*cur != 0) && (*cur != ' ') && (*cur != PATH_SEPARATOR))
 	    cur++;
 	if (cur != path) {
 	    paths[nbpaths] = xmlStrndup(path, cur - path);
@@ -2089,6 +2098,26 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
         }
     }
 #endif /* LIBXML_PUSH_ENABLED */
+#ifdef HAVE_SYS_MMAN_H
+    else if ((html) && (memory)) {
+	int fd;
+	struct stat info;
+	const char *base;
+	if (stat(filename, &info) < 0) 
+	    return;
+	if ((fd = open(filename, O_RDONLY)) < 0)
+	    return;
+	base = mmap(NULL, info.st_size, PROT_READ, MAP_SHARED, fd, 0) ;
+	if (base == (void *) MAP_FAILED)
+	    return;
+
+	doc = htmlReadMemory((char *) base, info.st_size, filename,
+	                     NULL, options);
+	    
+	munmap((char *) base, info.st_size);
+	close(fd);
+    }
+#endif
     else if (html) {
 	doc = htmlReadFile(filename, NULL, options);
     }
@@ -2203,6 +2232,7 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
 			                filename, NULL, options);
 	        
 	    munmap((char *) base, info.st_size);
+	    close(fd);
 #endif
 #ifdef LIBXML_VALID_ENABLED
 	} else if (valid) {
@@ -2280,8 +2310,10 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
     /*
      * shell interaction
      */
-    if (shell)  
+    if (shell) {
+        xmlXPathOrderDocElems(doc);
         xmlShell(doc, filename, xmlShellReadline, stdout);
+    }
 #endif
 #endif
 
@@ -2443,48 +2475,71 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
 		    write(1, result, len);
 		    xmlFree(result);
 		}
+
 	    } else
 #endif /* HAVE_SYS_MMAN_H */
 	    if (compress) {
 		xmlSaveFile(output ? output : "-", doc);
-	    }
-	    else if (encoding != NULL) {
-	        if ( format ) {
-		    ret = xmlSaveFormatFileEnc(output ? output : "-", doc,
-		                               encoding, 1);
+	    } else if (oldout) {
+	        if (encoding != NULL) {
+		    if ( format ) {
+			ret = xmlSaveFormatFileEnc(output ? output : "-", doc,
+						   encoding, 1);
+		    }
+		    else {
+			ret = xmlSaveFileEnc(output ? output : "-", doc,
+			                     encoding);
+		    }
+		    if (ret < 0) {
+			fprintf(stderr, "failed save to %s\n",
+				output ? output : "-");
+			progresult = XMLLINT_ERR_OUT;
+		    }
+		} else if (format) {
+		    ret = xmlSaveFormatFile(output ? output : "-", doc, 1);
+		    if (ret < 0) {
+			fprintf(stderr, "failed save to %s\n",
+				output ? output : "-");
+			progresult = XMLLINT_ERR_OUT;
+		    }
+		} else {
+		    FILE *out;
+		    if (output == NULL)
+			out = stdout;
+		    else {
+			out = fopen(output,"wb");
+		    }
+		    if (out != NULL) {
+			if (xmlDocDump(out, doc) < 0)
+			    progresult = XMLLINT_ERR_OUT;
+
+			if (output != NULL)
+			    fclose(out);
+		    } else {
+			fprintf(stderr, "failed to open %s\n", output);
+			progresult = XMLLINT_ERR_OUT;
+		    }
 		}
-		else {
-		    ret = xmlSaveFileEnc(output ? output : "-", doc, encoding);
-		}
-		if (ret < 0) {
-		    fprintf(stderr, "failed save to %s\n",
-		            output ? output : "-");
-		    progresult = XMLLINT_ERR_OUT;
-		}
-	    }
-	    else if (format) {
-		ret = xmlSaveFormatFile(output ? output : "-", doc, 1);
-		if (ret < 0) {
-		    fprintf(stderr, "failed save to %s\n",
-		            output ? output : "-");
-		    progresult = XMLLINT_ERR_OUT;
-		}
-	    }
-	    else {
-		FILE *out;
+	    } else {
+	        xmlSaveCtxtPtr ctxt;
+		int saveOpts = 0;
+
+                if (format)
+		    saveOpts |= XML_SAVE_FORMAT;
+
 		if (output == NULL)
-		    out = stdout;
-		else {
-		    out = fopen(output,"wb");
-		}
-		if (out != NULL) {
-		    if (xmlDocDump(out, doc) < 0)
-		        progresult = XMLLINT_ERR_OUT;
+		    ctxt = xmlSaveToFd(1, encoding, saveOpts);
+		else
+		    ctxt = xmlSaveToFilename(output, encoding, saveOpts);
 
-		    if (output != NULL)
-			fclose(out);
+		if (ctxt != NULL) {
+		    if (xmlSaveDoc(ctxt, doc) < 0) {
+			fprintf(stderr, "failed save to %s\n",
+				output ? output : "-");
+			progresult = XMLLINT_ERR_OUT;
+		    }
+		    xmlSaveClose(ctxt);
 		} else {
-		    fprintf(stderr, "failed to open %s\n", output);
 		    progresult = XMLLINT_ERR_OUT;
 		}
 	    }
@@ -2752,6 +2807,7 @@ static void showVersion(const char *name) {
     if (xmlHasFeature(XML_WITH_DEBUG)) fprintf(stderr, "Debug "); 
     if (xmlHasFeature(XML_WITH_DEBUG_MEM)) fprintf(stderr, "MemDebug "); 
     if (xmlHasFeature(XML_WITH_DEBUG_RUN)) fprintf(stderr, "RunDebug "); 
+    if (xmlHasFeature(XML_WITH_ZLIB)) fprintf(stderr, "Zlib ");
     fprintf(stderr, "\n");
 }
 
